@@ -34,7 +34,7 @@
 
 #ident "$Copyright: (c) 1980, 1990 Regents of the University of California. $"
 #ident "$Copyright: All rights reserved. $"
-#ident "$Id: quota.c,v 1.20 2006/05/13 01:05:24 jkar8572 Exp $"
+#ident "$Id: quota.c,v 1.24 2007/06/20 16:22:23 jkar8572 Exp $"
 
 /*
  * Disk quota reporting program.
@@ -43,6 +43,7 @@
 #include <sys/param.h>
 #include <getopt.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <pwd.h>
 #include <grp.h>
 #include <time.h>
@@ -71,6 +72,8 @@
 #define FL_NOWRAP 256
 #define FL_FSLIST 512
 #define FL_NUMNAMES 1024
+#define FL_NFSALL 2048
+#define FL_RAWGRACE 4096
 
 int flags, fmt = -1;
 char *progname;
@@ -78,9 +81,9 @@ char *progname;
 void usage(void)
 {
 	errstr( "%s%s%s%s%s",
-		_("Usage: quota [-guqvsw] [-l | -Q] [-i] [-F quotaformat]\n"),
-		_("\tquota [-qvsw] [-l | -Q] [-i] [-F quotaformat] -u username ...\n"),
-		_("\tquota [-qvsw] [-l | -Q] [-i] [-F quotaformat] -g groupname ...\n"),
+		_("Usage: quota [-guqvsw] [-l | [-Q | -A]] [-i] [-F quotaformat]\n"),
+		_("\tquota [-qvsw] [-l | [-Q | -A]] [-i] [-F quotaformat] -u username ...\n"),
+		_("\tquota [-qvsw] [-l | [-Q | -A]] [-i] [-F quotaformat] -g groupname ...\n"),
 		_("\tquota [-qvswugQ] [-F quotaformat] -f filesystem ...\n"),
 		_("\n\
 -u, --user                display quota for user\n\
@@ -91,12 +94,14 @@ void usage(void)
     --always-resolve      always try to translate name to id, even if it is\n\
 			  composed of only digits\n\
 -w, --no-wrap             do not wrap long lines\n\
+-p, --raw-grace           print grace time in seconds since epoch\n\
 -l, --local-only          do not query NFS filesystems\n\
 -Q, --quiet-refuse        do not print error message when NFS server does\n\
                           not respond\n\
 -i, --no-autofs           do not query autofs mountpoints\n\
 -F, --format=formatname   display quota of a specific format\n\
 -f, --filesystem-list     display quota information only for given filesystems\n\
+-A, --nfs-all             display quota for all NFS mountpoints\n\
 -h, --help                display this help message and exit\n\
 -V, --version             display version information and exit\n\n"));
 	fprintf(stderr, _("Bugs to: %s\n"), MY_EMAIL);
@@ -126,7 +131,7 @@ int showquotas(int type, qid_t id, int mntcnt, char **mnt)
 
 	time(&now);
 	id2name(id, type, name);
-	handles = create_handle_list(mntcnt, mnt, type, fmt, IOI_READONLY, ((flags & FL_NOAUTOFS) ? MS_NO_AUTOFS : 0) | ((flags & FL_LOCALONLY) ? MS_LOCALONLY : 0));
+	handles = create_handle_list(mntcnt, mnt, type, fmt, IOI_READONLY, ((flags & FL_NOAUTOFS) ? MS_NO_AUTOFS : 0) | ((flags & FL_LOCALONLY) ? MS_LOCALONLY : 0) | ((flags & FL_NFSALL) ? MS_NFS_ALL : 0));
 	qlist = getprivs(id, handles, !!(flags & FL_QUIETREFUSE));
 	over = 0;
 	for (q = qlist; q; q = q->dq_next) {
@@ -185,20 +190,41 @@ int showquotas(int type, qid_t id, int mntcnt, char **mnt)
 				printf("%s\n%15s", q->dq_h->qh_quotadev, "");
 			else
 				printf("%15s", q->dq_h->qh_quotadev);
-			if (bover)
-				difftime2str(q->dq_dqb.dqb_btime, timebuf);
+			if (!(flags & FL_RAWGRACE)) {
+				if (bover)
+					difftime2str(q->dq_dqb.dqb_btime, timebuf);
+				else
+					timebuf[0] = 0;
+			}
+			else {
+				if (bover)
+					sprintf(timebuf, "%Lu", (long long unsigned int)q->dq_dqb.dqb_btime);
+				else
+					strcpy(timebuf, "0");
+			}
 			space2str(toqb(q->dq_dqb.dqb_curspace), numbuf[0], !!(flags & FL_SMARTSIZE));
 			space2str(q->dq_dqb.dqb_bsoftlimit, numbuf[1], !!(flags & FL_SMARTSIZE));
 			space2str(q->dq_dqb.dqb_bhardlimit, numbuf[2], !!(flags & FL_SMARTSIZE));
 			printf(" %7s%c %6s %7s %7s", numbuf[0], bover ? '*' : ' ', numbuf[1],
-			       numbuf[2], bover ? timebuf : "");
-			if (iover)
-				difftime2str(q->dq_dqb.dqb_itime, timebuf);
+			       numbuf[2], timebuf);
+
+			if (!(flags & FL_RAWGRACE)) {
+				if (iover)
+					difftime2str(q->dq_dqb.dqb_itime, timebuf);
+				else
+					timebuf[0] = 0;
+			}
+			else {
+				if (iover)
+					sprintf(timebuf, "%Lu", (long long unsigned int)q->dq_dqb.dqb_itime);
+				else
+					strcpy(timebuf, "0");
+			}
 			number2str(q->dq_dqb.dqb_curinodes, numbuf[0], !!(flags & FL_SMARTSIZE));
 			number2str(q->dq_dqb.dqb_isoftlimit, numbuf[1], !!(flags & FL_SMARTSIZE));
 			number2str(q->dq_dqb.dqb_ihardlimit, numbuf[2], !!(flags & FL_SMARTSIZE));
 			printf(" %7s%c %6s %7s %7s\n", numbuf[0], iover ? '*' : ' ', numbuf[1],
-			       numbuf[2], iover ? timebuf : "");
+			       numbuf[2], timebuf);
 			continue;
 		}
 	}
@@ -223,19 +249,21 @@ int main(int argc, char **argv)
 		{ "verbose", 0, NULL, 'v' },
 		{ "human-readable", 0, NULL, 's' },
 		{ "always-resolve", 0, NULL, 256 },
+		{ "raw-grace", 0, NULL, 'p' },
 		{ "local-only", 0, NULL, 'l' },
 		{ "no-autofs", 0, NULL, 'i' },
 		{ "quiet-refuse", 0, NULL, 'Q' },
 		{ "format", 1, NULL, 'F' },
 		{ "no-wrap", 0, NULL, 'w' },
 		{ "filesystem-list", 0, NULL, 'f' },
+		{ "all-nfs", 0, NULL, 'A' },
 		{ NULL, 0, NULL, 0 }
 	};
 
 	gettexton();
 	progname = basename(argv[0]);
 
-	while ((ret = getopt_long(argc, argv, "guqvsVliQF:wf", long_opts, NULL)) != -1) {
+	while ((ret = getopt_long(argc, argv, "guqvsVliQF:wfAp", long_opts, NULL)) != -1) {
 		switch (ret) {
 		  case 'g':
 			  flags |= FL_GROUP;
@@ -256,6 +284,9 @@ int main(int argc, char **argv)
 		  case 's':
 			  flags |= FL_SMARTSIZE;
 			  break;
+		  case 'p':
+			  flags |= FL_RAWGRACE;
+			  break;
 		  case 256:
 			  flags |= FL_NUMNAMES;
 			  break;
@@ -274,6 +305,9 @@ int main(int argc, char **argv)
 		  case 'f':
 			  flags |= FL_FSLIST;
 			  break;
+		  case 'A':
+			  flags |= FL_NFSALL;
+			  break;
 		  case 'V':
 			  version();
 			  exit(0);
