@@ -3,12 +3,12 @@
  *	Utility to check disk quotas
  *
  *	Some parts of this utility are copied from old quotacheck by
- *	Marco van Wieringen <mvw@planets.elm.net> and Edvard Tuinder <ed@elm.ent>
+ *	Marco van Wieringen <mvw@planets.elm.net> and Edvard Tuinder <ed@elm.net>
  * 
  *	New quota format implementation - Jan Kara <jack@suse.cz> - Sponsored by SuSE CR
  */
 
-#ident "$Id: quotacheck.c,v 1.51 2006/05/13 01:05:24 jkar8572 Exp $"
+#ident "$Id: quotacheck.c,v 1.53 2007/02/21 13:51:25 jkar8572 Exp $"
 
 #include <dirent.h>
 #include <stdio.h>
@@ -1039,11 +1039,11 @@ static void check_all(void)
 			continue;
 		}
 		cfmt = fmt;
-		if (uwant && hasquota(mnt, USRQUOTA))
+		if (uwant && hasquota(mnt, USRQUOTA, 0))
 			ucheck = 1;
 		else
 			ucheck = 0;
-		if (gwant && hasquota(mnt, GRPQUOTA))
+		if (gwant && hasquota(mnt, GRPQUOTA, 0))
 			gcheck = 1;
 		else
 			gcheck = 0;
