@@ -10,7 +10,7 @@
  * 
  * Author:  Marco van Wieringen <mvw@planets.elm.net>
  *
- * Version: $Id: warnquota.c,v 1.26 2006/05/13 01:05:24 jkar8572 Exp $
+ * Version: $Id: warnquota.c,v 1.28 2007/05/29 14:17:56 jkar8572 Exp $
  *
  *          This program is free software; you can redistribute it and/or
  *          modify it under the terms of the GNU General Public License as
@@ -112,6 +112,7 @@ struct configparams {
 	int ldap_is_setup; /* 0 */
 	char ldap_host[CNF_BUFFER];
 	int ldap_port;
+	char ldap_uri[CNF_BUFFER];
 	char ldap_binddn[CNF_BUFFER];
 	char ldap_bindpw[CNF_BUFFER];
 	char ldap_basedn[CNF_BUFFER];
@@ -161,28 +162,55 @@ static void wc_exit(int ex_stat)
 {
 #ifdef USE_LDAP_MAIL_LOOKUP
 	if(ldapconn != NULL)
+#ifdef USE_LDAP_23
+		ldap_unbind_ext(ldapconn, NULL, NULL);
+#else
 		ldap_unbind(ldapconn);
+#endif
 #endif
 	exit(ex_stat);
 }
 
 #ifdef USE_LDAP_MAIL_LOOKUP
+#ifdef USE_LDAP_23
+void ldap_perror(LDAP *ld, char *s)
+{
+	int err;
+
+	ldap_get_option(ld, LDAP_OPT_RESULT_CODE, &err);
+	errstr(_("%s: %s\n"), s, ldap_err2string(err));
+}
+#endif
+
 static int setup_ldap(struct configparams *config)
 {
 	int ret;
+#ifdef USE_LDAP_23
+	struct berval cred = { .bv_val = config->ldap_bindpw,
+			       .bv_len = strlen(config->ldap_bindpw) };
+#endif
 
+#ifdef USE_LDAP_23
+	ldap_initialize(&ldapconn, config->ldap_uri);
+#else
 	ldapconn = ldap_init(config->ldap_host, config->ldap_port);
+#endif
+
 	if(ldapconn == NULL) {
 		ldap_perror(ldapconn, "ldap_init");
-		return(-1);
+		return -1;
 	}
 
+#ifdef USE_LDAP_23
+	ret = ldap_sasl_bind_s(ldapconn, config->ldap_binddn, LDAP_SASL_SIMPLE, &cred, NULL, NULL, NULL);
+#else
 	ret = ldap_bind_s(ldapconn, config->ldap_binddn, config->ldap_bindpw, LDAP_AUTH_SIMPLE);
+#endif
 	if(ret < 0) {
 		ldap_perror(ldapconn, "ldap_bind");
-		return(-1);
+		return -1;
 	}
-	return(0);
+	return 0;
 }
 		
 #endif
@@ -392,9 +420,15 @@ static int mail_user(struct offenderlist *offender, struct configparams *config)
 				/* search for the offender_name in ldap */
 				snprintf(searchbuf, 256, "(%s=%s)", config->ldap_search_attr, 
 					offender->offender_name);
+#ifdef USE_LDAP_23
+				ret = ldap_search_ext_s(ldapconn, config->ldap_basedn, 
+					LDAP_SCOPE_SUBTREE, searchbuf,
+					NULL, 0, NULL, NULL, NULL, 0, &result);
+#else
 				ret = ldap_search_s(ldapconn, config->ldap_basedn, 
 					LDAP_SCOPE_SUBTREE, searchbuf,
 					NULL, 0, &result);
+#endif
 				if(ret < 0) {
 					errstr(_("Error with %s.\n"), offender->offender_name);
 					ldap_perror(ldapconn, "ldap_search");
@@ -681,6 +715,8 @@ static int readconfigfile(const char *filename, struct configparams *config)
 
 #ifdef USE_LDAP_MAIL_LOOKUP
 	config->ldap_port = config->ldap_is_setup = 0;
+	config->ldap_host[0] = 0;
+	config->ldap_uri[0] = 0;
 #endif
 
 	if (!(fp = fopen(filename, "r"))) {
@@ -774,7 +810,7 @@ static int readconfigfile(const char *filename, struct configparams *config)
 					goto cc_parse_err;
 				if (str2timeunits(num, unit, &config->cc_before) < 0) {
 cc_parse_err:
-					errstr(_("Cannot parse time at CC_BEFORE variable (line %d).\n"), line);
+					die(1, _("Cannot parse time at CC_BEFORE variable (line %d).\n"), line);
 				}
 			}
 #ifdef USE_LDAP_MAIL_LOOKUP
@@ -782,6 +818,8 @@ cc_parse_err:
 				sstrncpy(config->ldap_host, value, CNF_BUFFER);
 			else if (!strcmp(var, "LDAP_PORT"))
 				config->ldap_port = (int)strtol(value, NULL, 10);
+			else if (!strcmp(var, "LDAP_URI"))
+				sstrncpy(config->ldap_uri, value, CNF_BUFFER);
 			else if(!strcmp(var, "LDAP_BINDDN"))
 				sstrncpy(config->ldap_binddn, value, CNF_BUFFER);
 			else if(!strcmp(var, "LDAP_BINDPW"))
@@ -803,6 +841,17 @@ cc_parse_err:
 	}
 	if (bufpos)
 		errstr(_("Unterminated last line, ignoring\n"));
+#ifdef USE_LDAP_MAIL_LOOKUP
+#ifdef USE_LDAP_23
+	if (!config->ldap_uri[0]) {
+		snprintf(config->ldap_uri, CNF_BUFFER, "ldap://%s:%d", config->ldap_host, config->ldap_port);
+		errstr(_("LDAP library version >= 2.3 detected. Please use LDAP_URI instead of hostname and port.\nGenerated URI %s\n"), config->ldap_uri);
+	}
+#else
+	if (config->ldap_uri[0])
+		die(1, _("LDAP library does not support ldap_initialize() but URI is specified."));
+#endif
+#endif
 	fclose(fp);
 
 	return 0;
